"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Provides utility methods for navigating TreeViews.
 */
var TreeViewNavigationUtils = /** @class */ (function () {
    function TreeViewNavigationUtils() {
    }
    /**
     * Normalizes the keyboard event by preventing propogation and disabling
     * the scrolling effect on non-tab key presses.
     */
    TreeViewNavigationUtils.normalizeKeyboardEventBehaviour = function (event) {
        // Don't block input on inputs
        if (event.target && event.target.tagName.toUpperCase() === 'INPUT') {
            return;
        }
        if (event.keyCode !== 13 && event.keyCode !== 9) {
            // The enter/esc keys are the only exception, as this is especially used to submit forms
            event.stopPropagation();
        }
        if (event.keyCode !== 9) {
            // Allow tabbing out
            event.preventDefault();
        }
    };
    /**
     * In a TreeView, handles up, down, home, and end key presses to behave according
     * to the wai-aria spec defined in https://www.w3.org/TR/wai-aria-practices-1.1/#TreeView
     */
    TreeViewNavigationUtils.handleMovementKeyboardEvent = function (thisId, treeViewId, event, onAccessibleSelected) {
        var navigatables;
        var elem = undefined;
        switch (event.keyCode) {
            case 35:
                // End
                navigatables = TreeViewNavigationUtils.getAllNavigatables(treeViewId);
                if (navigatables.length > 0) {
                    elem = navigatables[navigatables.length - 1];
                    elem.focus();
                }
                break;
            case 36:
                // Home
                navigatables = TreeViewNavigationUtils.getAllNavigatables(treeViewId);
                if (navigatables.length > 0) {
                    elem = navigatables[0];
                    elem.focus();
                }
                break;
            case 38:
                // Up arrow
                elem = TreeViewNavigationUtils.getNavigatableWithOffset(thisId, treeViewId, -1);
                if (!!elem) {
                    elem.focus();
                }
                break;
            case 40:
                // Down arrow
                elem = TreeViewNavigationUtils.getNavigatableWithOffset(thisId, treeViewId, 1);
                if (!!elem) {
                    elem.focus();
                }
                break;
            default:
                break;
        }
        if (elem) {
            var id = elem.getAttribute('data-id');
            if (id) {
                onAccessibleSelected(id);
            }
        }
    };
    /**
     * Given a non-zero integer offset, finds the navigatable within the same TreeView that is offsetted from the
     * element with the id attribute matching thisId.
     */
    TreeViewNavigationUtils.getNavigatableWithOffset = function (thisId, treeViewId, offset) {
        if (offset === 0 || Math.floor(offset) !== offset) {
            // We expect a non-zero integer offset
            return undefined;
        }
        var navigatables = TreeViewNavigationUtils.getAllNavigatables(treeViewId);
        // Depending on the offset, we don't have to check all elements. For example, if we're going
        // backwards by n, we don't need to check the first n elements etc.
        var startIndex = offset < 0 ? 0 - offset : 0;
        var endIndex = offset > 0 ? navigatables.length - offset : navigatables.length;
        for (var i = startIndex; i < endIndex; i++) {
            var currentElem = navigatables[i];
            if (currentElem.getAttribute('data-id') === thisId) {
                return navigatables[i + offset];
            }
        }
        return undefined;
    };
    /**
     * Given a TreeView id, gets all navigatable (focusable) elements in that TreeView.
     */
    TreeViewNavigationUtils.getAllNavigatables = function (treeViewId) {
        return document.querySelectorAll("[data-treeviewid=" + treeViewId + "]");
    };
    return TreeViewNavigationUtils;
}());
exports.TreeViewNavigationUtils = TreeViewNavigationUtils;
//# sourceMappingURL=treeViewNavigationUtils.js.map