"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var oneNoteItemUtils_1 = require("./oneNoteItemUtils");
/**
 * Responsible for updating the notebook list object in a controlled manner.
 * In particular, preserves expanded state. There is no guarantee that the
 * internal notebook list object * will be modified or replaced. Logic relying
 * on this class should use the getter as the most recent source of truth.
 */
var NotebookListUpdater = /** @class */ (function () {
    function NotebookListUpdater(initialNotebooks) {
        this.notebooks = initialNotebooks;
    }
    /**
     * Gets the notebook list
     */
    NotebookListUpdater.prototype.get = function () {
        return this.notebooks;
    };
    /**
     * Updates the internal notebooks with a more recent version of the notebooks,
     * returned from the API.
     * @param apiNotebooks The API notebooks to update the internal notebooks with.
     */
    NotebookListUpdater.prototype.updateNotebookList = function (newNotebooks) {
        var oldNotebooks = this.notebooks;
        this.notebooks = newNotebooks;
        if (oldNotebooks.length === 0) {
            return;
        }
        var _loop_1 = function (newNotebook) {
            var originalNotebook = oldNotebooks.find(function (notebook) { return notebook.id === newNotebook.id; });
            if (!!originalNotebook) {
                this_1.preserveSectionParent(originalNotebook, newNotebook);
            }
        };
        var this_1 = this;
        // TODO (machiam) cut down on repeat code after we have UTs
        for (var _i = 0, newNotebooks_1 = newNotebooks; _i < newNotebooks_1.length; _i++) {
            var newNotebook = newNotebooks_1[_i];
            _loop_1(newNotebook);
        }
    };
    NotebookListUpdater.prototype.addNotebook = function (newNotebook) {
        // Always prepend
        var newNotebooks = [newNotebook].concat(this.notebooks);
        this.notebooks = newNotebooks;
    };
    NotebookListUpdater.prototype.addSection = function (newSection) {
        var loneParent = newSection.parent;
        var parentInHierarchy = oneNoteItemUtils_1.OneNoteItemUtils.find(this.notebooks, function (item) { return item.id === loneParent.id; });
        if (parentInHierarchy && parentInHierarchy.sections) {
            // Establish two-way reference
            parentInHierarchy.sections = [newSection].concat(parentInHierarchy.sections);
            newSection.parent = parentInHierarchy;
        }
    };
    NotebookListUpdater.prototype.preserveSectionParent = function (original, next) {
        // Preserve properties we want to keep from the original object ...
        next.expanded = original.expanded;
        if (!original.sections || !original.sectionGroups ||
            !next.sections || !next.sectionGroups) {
            return;
        }
        var _loop_2 = function (newSectionGroup) {
            var originalSectionGroup = original.sectionGroups.find(function (sg) { return sg.id === newSectionGroup.id; });
            if (!!originalSectionGroup) {
                this_2.preserveSectionParent(originalSectionGroup, newSectionGroup);
            }
        };
        var this_2 = this;
        // ... then recurse through the children
        for (var _i = 0, _a = next.sectionGroups; _i < _a.length; _i++) {
            var newSectionGroup = _a[_i];
            _loop_2(newSectionGroup);
        }
        var _loop_3 = function (newSection) {
            var originalSection = original.sections.find(function (section) { return section.id === newSection.id; });
            if (!!originalSection) {
                newSection.expanded = originalSection.expanded;
            }
        };
        // TODO (machiam) cut down on repeat code after we have UTs
        for (var _b = 0, _c = next.sections; _b < _c.length; _b++) {
            var newSection = _c[_b];
            _loop_3(newSection);
        }
    };
    /**
     * Updates the internal notebooks' section with a more recent version of its
     * sections.
     * @param parentSectionId Parent section id.
     * @param apiPages The API pages to update the internal notebooks' matching section with.
     */
    NotebookListUpdater.prototype.updatePages = function (parentSectionId, newPages) {
        if (!this.notebooks) {
            return;
        }
        var sectionRef = this.getSectionRefFromNotebooks(parentSectionId, this.notebooks);
        if (!sectionRef) {
            return;
        }
        sectionRef.pages = newPages;
    };
    NotebookListUpdater.prototype.getSectionRefFromNotebooks = function (sectionId, notebooks) {
        for (var _i = 0, notebooks_1 = notebooks; _i < notebooks_1.length; _i++) {
            var notebook = notebooks_1[_i];
            var sectionRef = this.getSectionRefFromSectionParent(sectionId, notebook);
            if (!!sectionRef) {
                return sectionRef;
            }
        }
        // We couldn't find the section in this subtree
        return undefined;
    };
    NotebookListUpdater.prototype.getSectionRefFromSectionParent = function (sectionId, sectionParent) {
        if (!sectionParent.sections || !sectionParent.sectionGroups) {
            return undefined;
        }
        // Search this parent's sections for the matching id ...
        for (var _i = 0, _a = sectionParent.sections; _i < _a.length; _i++) {
            var childSection = _a[_i];
            if (childSection.id === sectionId) {
                return childSection;
            }
        }
        // ... then recurse through section groups
        for (var _b = 0, _c = sectionParent.sectionGroups; _b < _c.length; _b++) {
            var childSectionParent = _c[_b];
            var sectionRef = this.getSectionRefFromSectionParent(sectionId, childSectionParent);
            if (!!sectionRef) {
                return sectionRef;
            }
        }
        // We couldn't find the section in this subtree
        return undefined;
    };
    return NotebookListUpdater;
}());
exports.NotebookListUpdater = NotebookListUpdater;
//# sourceMappingURL=notebookListUpdater.js.map