"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var oneNoteApiResponseTransformer_1 = require("../oneNoteDataStructures/oneNoteApiResponseTransformer");
/**
 * Implements OneNoteDataProvider with external calls to OneNote's API.
 */
var OneNoteApiDataProvider = /** @class */ (function () {
    function OneNoteApiDataProvider(api) {
        this.api = api;
        this.responseTransformer = new oneNoteApiResponseTransformer_1.OneNoteApiResponseTransformer();
    }
    OneNoteApiDataProvider.prototype.createNotebook = function (name) {
        var _this = this;
        // tslint:disable-next-line:no-any
        return this.api.createNotebook(name).then(function (responsePackage) {
            return Promise.resolve(_this.responseTransformer.transformNotebook(responsePackage.parsedResponse));
        });
    };
    OneNoteApiDataProvider.prototype.createSectionUnderNotebook = function (parent, name) {
        var _this = this;
        // tslint:disable-next-line:no-any
        return this.api.createSection(parent.id, name).then(function (responsePackage) {
            return Promise.resolve(_this.responseTransformer.transformSection(responsePackage.parsedResponse, parent));
        });
    };
    OneNoteApiDataProvider.prototype.createSectionUnderSectionGroup = function (parent, name) {
        var _this = this;
        // tslint:disable-next-line:no-any
        return this.api.createSection(parent.id, name).then(function (responsePackage) {
            return Promise.resolve(_this.responseTransformer.transformSection(responsePackage.parsedResponse, parent));
        });
    };
    OneNoteApiDataProvider.prototype.getNotebooks = function (expands, excludeReadOnlyNotebooks) {
        var _this = this;
        // tslint:disable-next-line:no-any
        return this.api.getNotebooksWithExpandedSections(expands, excludeReadOnlyNotebooks).then(function (responsePackage) {
            return Promise.resolve(_this.responseTransformer.transformNotebooks(responsePackage.parsedResponse.value));
        });
    };
    OneNoteApiDataProvider.prototype.getNotebookBySelfUrl = function (selfUrl, expands) {
        var _this = this;
        return this.api.getNotebookBySelfUrl(selfUrl, expands).then(function (response) {
            return _this.responseTransformer.transformNotebook(response.parsedResponse);
        });
    };
    OneNoteApiDataProvider.prototype.getPages = function (section) {
        var _this = this;
        // tslint:disable-next-line:no-any
        return this.api.getPages({ sectionId: section.id }).then(function (responsePackage) {
            return Promise.resolve(_this.responseTransformer.transformPages(responsePackage.parsedResponse.value, section));
        });
    };
    OneNoteApiDataProvider.prototype.getSpNotebooks = function () {
        var _this = this;
        return this.api.performApiCall("https://www.onenote.com/api/v1.0/me/notes/notebooks/getrecentnotebooks(includePersonalNotebooks=false)", undefined /* data */, undefined /* content type */, 'GET' /* http method */, true /* isFullUrl */).then(function (response) {
            // tslint:disable-next-line:no-any
            var serviceSharedNotebooks = response.parsedResponse.value;
            var sharedNotebooks = [];
            for (var i = 0; i < serviceSharedNotebooks.length; i++) {
                if (_this.isOdbNotebook(serviceSharedNotebooks[i])) {
                    sharedNotebooks.push({
                        id: '',
                        parent: undefined,
                        name: _this.getSharedNotebookName(serviceSharedNotebooks[i]),
                        webUrl: serviceSharedNotebooks[i].links.oneNoteWebUrl.href,
                        sourceService: serviceSharedNotebooks[i].sourceService,
                        expanded: false,
                        sections: [],
                        sectionGroups: [],
                        apiUrl: '',
                        lastModifiedTime: serviceSharedNotebooks[i].lastAccessedTime
                    });
                }
            }
            return sharedNotebooks;
        });
    };
    // tslint:disable-next-line:no-any
    OneNoteApiDataProvider.prototype.isOdbNotebook = function (notebook) {
        if (notebook.sourceService === 'OneDriveForBusiness') {
            return true;
        }
        if (notebook.sourceService === 'Unknown') {
            var webUrl = notebook.links.oneNoteWebUrl.href;
            var segments = webUrl.split('/');
            if (segments.length > 2) {
                // This is somewhat naive, but we want to filter out ppe and live
                var domain = segments[2];
                return domain.indexOf('.spoppe.') < 0 && domain !== 'd.docs.live.net';
            }
        }
        return false;
    };
    OneNoteApiDataProvider.prototype.getSharedNotebookName = function (sharedNotebook) {
        var name = sharedNotebook.name;
        var webUrl = sharedNotebook.links && sharedNotebook.links.oneNoteWebUrl && sharedNotebook.links.oneNoteWebUrl.href;
        if (!name && !webUrl) {
            return '';
        }
        // We infer the name from the URL if name is not provided
        if (!name && webUrl) {
            var urlParts = webUrl.split('/');
            // There might be a trailing slash we would like to account for. We pop twice. pop() returns undefined if array is empty
            var lastPart = urlParts.pop() || urlParts.pop();
            if (lastPart) {
                name = decodeURI(lastPart);
            }
        }
        var trimmedName = name && name.trim();
        if (trimmedName) {
            return trimmedName;
        }
        // We guess that the last url part is the name
        var splitUrl = webUrl.split('/');
        var last = splitUrl[splitUrl.length - 1];
        return decodeURIComponent(last);
    };
    OneNoteApiDataProvider.prototype.addQueryParamComponent = function (originalUrl, queryParamComponent) {
        if (originalUrl.indexOf('?') === -1) {
            return originalUrl + '?' + queryParamComponent;
        }
        else {
            return originalUrl + '&' + queryParamComponent;
        }
    };
    OneNoteApiDataProvider.prototype.getSpNotebookProperties = function (spNotebook, expands, excludeReadOnlyNotebooks) {
        var _this = this;
        return this.getNotebookSelfUrlFromSpUrl(spNotebook.webUrl).then(function (selfUrl) {
            return _this.api.performApiCall(_this.addQueryParamComponent(selfUrl, _this.getExpands(expands)), undefined /* data */, undefined /* content type */, 'GET' /* http method */, true /* isFullUrl */).then(function (response) {
                var notebook = response.parsedResponse;
                if (notebook) {
                    spNotebook.apiUrl = notebook.self;
                    var spSections = notebook.sections.map(function (section) { return _this.responseTransformer.transformSection(section, spNotebook); });
                    var spSectionGroups = notebook.sectionGroups.map(function (sectionGroup) { return _this.responseTransformer.transformSectionGroup(sectionGroup, spNotebook); });
                    return {
                        id: notebook.id,
                        spSectionGroups: spSectionGroups,
                        spSections: spSections
                    };
                }
                throw new Error('Could not find notebook in get notebooks expands request');
            });
        });
    };
    OneNoteApiDataProvider.prototype.getNotebookSelfUrlFromSpUrl = function (spNotebookUrl) {
        return this.api.performApiCall("https://www.onenote.com/api/beta/me/notes/notebooks/GetNotebookFromWebUrl()", JSON.stringify({ webUrl: spNotebookUrl }) /* data */, 'application/json' /* content type */, 'POST' /* http method */, true /* isFullUrl */).then(function (response) {
            var notebook = response.parsedResponse;
            if (notebook && notebook.self) {
                return notebook.self;
            }
            else {
                throw new Error('Could not find notebook in GetNotebookFromWebUrl response');
            }
        });
    };
    OneNoteApiDataProvider.prototype.getExpands = function (expands) {
        if (!expands || expands <= 0) {
            return '';
        }
        var s = '$expand=sections,sectionGroups';
        return expands === 1 ? s : s + "(" + this.getExpands(expands - 1) + ")";
    };
    return OneNoteApiDataProvider;
}());
exports.OneNoteApiDataProvider = OneNoteApiDataProvider;
//# sourceMappingURL=oneNoteApiDataProvider.js.map